@@ -1,17 +1,19 @@
-#-----------------------#
-  package DBIx::Simple;
-#-----------------------#
+package DBIx::Simple;
+
 use DBI;
+use Carp;
 use strict;
+use vars qw($VERSION %results);
 
-our $VERSION = '0.02';
+my $omniholder = '(??)';
 
-our %results;
+$VERSION = '0.03';
 
 sub connect {
     my ($class, @arguments) = @_;
     my $self = {
-	dbi => DBI->connect(@arguments)
+	dbi => DBI->connect(@arguments),
+	omniholder => '(??)',
     };
     return undef unless $self->{dbi};
     return bless $self, $class;
@@ -22,17 +24,30 @@ sub disconnect {
     $self->{dbi}->disconnect() if $self->{dbi};
 }
 
+sub omniholder {
+    my ($self, $value) = @_;
+    return $self->{omniholder} = $value if @_ > 1;
+    return $self->{omniholder};
+}
+
 sub query {
     my ($self, $query, @binds) = @_;
     $self->{success} = 0;
     
+    if (defined $self->{omniholder} and length $self->{omniholder}) {
+	$query =~ s[\Q$self->{omniholder}\E]
+	           { '(' . join(', ', ('?') x @binds) . ')' }e;
+	croak 'There can be only one omniholder'
+	    if $query =~ /\Q$self->{omniholder}\E/;
+    }
+    
     my $sth = $self->{dbi}->prepare($query) or do {
-	$self->{reason} = 'Prepare failed';
+	$self->{reason} = "Prepare failed ($DBI::errstr)";
 	return DBIx::Simple::Dummy->new();
     };
     
     $sth->execute(@binds) or do {
-	$self->{reason} = 'Execute failed';
+	$self->{reason} = "Execute failed ($DBI::errstr)";
 	return DBIx::Simple::Dummy->new();
     };
 
@@ -45,13 +60,11 @@ sub query {
 }
 
 sub commit {
-    my ($self) = @_;
-    $self->{dbi}->commit();
+    $_[0]->{dbi}->commit();
 }
 
 sub rollback {
-    my ($self) = @_;
-    $self->{dbi}->rollback();
+    $_[0]->{dbi}->rollback();
 }
 
 sub DESTROY {
@@ -60,9 +73,7 @@ sub DESTROY {
     $self->disconnect();
 }
 
-#------------------------------#
-  package DBIx::Simple::Dummy;
-#------------------------------#
+package DBIx::Simple::Dummy;
 use strict;
 
 sub new      { bless \my $dummy, shift }
@@ -74,6 +85,8 @@ use strict;
 
 sub new {
     my ($class, $db, $sth) = @_;
+    # $db should be the stringified object, as a real reference
+    # would increase the reference count.
     my $self = {
 	db  => $db,
 	sth => $sth
@@ -82,28 +95,24 @@ sub new {
 }
 
 sub list {
-    my ($self) = @_;
-    return $self->{sth}->fetchrow_array;
+    return $_[0]->{sth}->fetchrow_array if wantarray;
+    return($_[0]->{sth}->fetchrow_array)[0];
 }
 
 sub array {
-    my ($self) = @_;
-    return $self->{sth}->fetchrow_arrayref;
+    return $_[0]->{sth}->fetchrow_arrayref;
 }
 
 sub hash {
-    my ($self) = @_;
-    return $self->{sth}->fetchrow_hashref;
+    return $_[0]->{sth}->fetchrow_hashref;
 }
 
 sub flat {
-    my ($self) = @_;
-    return map @$_, $self->arrays;
+    return map @$_, $_[0]->arrays;
 }
 
 sub arrays {
-    my ($self) = @_;
-    return @{ $self->{sth}->fetchall_arrayref };
+    return @{ $_[0]->{sth}->fetchall_arrayref };
 }
 
 sub hashes {
@@ -158,9 +167,7 @@ sub DESTROY {
     $self->{sth} = undef;
 }
 
-'Spirit moves through all things';
-
-__END__
+1;
 
 =head1 NAME
 
@@ -178,19 +185,27 @@ DBIx::Simple - An easy-to-use, object oriented interface to DBI
 	{ PrintError => 1 }            # Additional options
     );
 
+    # Abstracted example: $db->query($query, @variables)->what_you_want;
+
     #### SIMPLE QUERIES
 
-    $db->query('DELETE FROM foo');
-    die "$db->{reason} ($DBI::errstr)" if not $db->{success};
+    $db->query('DELETE FROM foo WHERE id=?', $id);
+    die $db->{reason} if not $db->{success};
 
     for (1..100) {
 	$db->query(
-	    'INSERT INTO foo VALUES (?, ?)',
+	    'INSERT INTO randomvalues VALUES (?, ?)',
 	    int rand(10),
 	    int rand(10)
 	);
     }
 
+    $db->query(
+	'INSERT INTO sometable VALUES (??)',
+	$first, $second, $third, $fourth, $fifth, $sixth
+    );
+    # (??) is expanded to (?, ?, ?, ?, ?, ?) automatically
+
     #### SINGLE ROW QUERIES
 
     my ($two)          = $db->query('SELECT 1 + 1')->list;
@@ -198,29 +213,29 @@ DBIx::Simple - An easy-to-use, object oriented interface to DBI
 
     #### FETCHING ALL IN ONE GO
 
-    my @names = $db->query('SELECT name FROM people WHERE foo > 5')->flat;
+    my @names = $db->query('SELECT name FROM people WHERE id > 5')->flat;
 
-    for my $row ($db->query('SELECT field1, field2 FROM foo')->arrays) {
-	print "--> $row->[0], $row->[1]\n";
+    for my $row ($db->query('SELECT name, email FROM people')->arrays) {
+	print "Name: $row->[0], Email: $row->[1]\n";
     }
 
-    for my $row ($db->query('SELECT field1, field2 FROM foo')->hashes) {
-	print "--> $row->{field1}, $row->{field2}\n";
+    for my $row ($db->query('SELECT name, email FROM people')->hashes) {
+	print "Name: $row->{name}, Email: $row->{email}\n";
     }
 
     #### FETCHING ONE ROW AT A TIME
 
     {
-	my $result = $db->query('SELECT field1, field2 FROM foo');
+	my $result = $db->query('SELECT name, email FROM people');
 	while (my $row = $result->array) {
-	    print "--> $row->[0], $row->[1]\n";
+	    print "Name: $row->[0], Email: $row->[1]\n";
 	}
     }
 
     {
-	my $result = $db->query('SELECT field1, field2 FROM foo');
+	my $result = $db->query('SELECT name, email FROM people');
 	while (my $row = $result->hash) {
-	    print "--> $row->{field1}, $row->{field2}\n";
+	    print "Name: $row->{field1}, Email: $row->{field2}\n";
 	}
     }
 
@@ -263,7 +278,7 @@ objects of which all methods return undef.
 
 =over 10
 
-=item C<DBIx::Simple-E<gt>connect( ... )>
+=item C<< DBIx::Simple->connect( ... ) >>
 
 This argument takes the exact arguments a normal DBI->connect()
 would take. It's the constructor method, and it returns a new
@@ -272,9 +287,23 @@ DBIx::Simple object.
 =item C<query($query, @values)>
 
 This calls DBI's prepare() and execute() methods, passing the values
-along to replace placeholders.  query() returns a new
+along to replace C<?> placeholders.  query() returns a new
 DBIx::Simple::Result object (or DBIx::Simple::Dummy), that can be
-used immediately to get data out of it.
+used immediately to get data out of it. You should always use
+placeholders instead of the variables themselves, as DBI will
+automatically quote and escape the values.
+
+DBIx::Simple provides an omniholder placeholder that will expand to
+C<(?, ?, ...)> with as many question marks as @values. There can be
+only one omniholder, and since it uses all given values, you shouldn't
+combine it with normal placeholders. This feature was inspired by the
+EZDBI module.
+
+=item C<omniholder($new_value)>
+
+This sets the omniholder string. Use C<undef> or an empty string to
+disable this feature. Please note that the given $new_value is not a
+regex. The default omniholder is C<(??)>.
 
 =item C<commit>, C<rollback>
 
@@ -285,7 +314,7 @@ These just call the DBI methods and Do What You Mean.
 Does What You Mean. Also note that the connection is automatically
 terminated when the object is destroyed (C<undef $db> to do so
 explicitly), and that all statements are also finished when the
-object is destoryed. disconnect() Does not destory the object.
+object is destroyed. disconnect() Does not destroy the object.
 
 =back
 
@@ -304,7 +333,8 @@ to find out what the arguments to new() are.
 =item C<list>
 
 list() Returns a list of elements in a single row. This is like a
-dereferenced C<$result->array()>.
+dereferenced C<$result->array()>. In scalar context, returns only the
+first value of the row.
 
 =item C<array> and C<hash>
 
@@ -315,7 +345,7 @@ fetchrow_hashref is used.
 =item C<flat>
 
 flat() Returns a list of all returned fields, flattened. This can be
-very useful if you select a single column. Consider flat to be 
+very useful if you select a single column. Consider to be 
 list()'s plural.
 
 =item C<arrays> and C<hashes>
@@ -339,13 +369,14 @@ values per row (other values will be discarded).
 
 =item C<rows>
 
-Returns the number of rows.
+Returns the number of rows. This function calls DBI's rows method,
+and may not do what you want. See L<DBI> for a good explanation.
 
 =item finish?
 
 There is no finish method. To finish the statement, just let the
-object go out of scope (you should always use "C<my>", and
-"C<use strict>") or destroy it explicitly using C<undef $result>.
+object go out of scope (you should always use C<my>, and
+C<use strict>) or destroy it explicitly using C<undef $result>.
 
 =back
 
